/*! litejs.com/MIT-LICENSE.txt */



/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



!function(exports, Object) {
	var P = "prototype"
	, A = Array[P], F = Function[P], S = String[P]
	, hasOwn = Object.prototype.hasOwnProperty
	, slice = F.call.bind(A.slice)
	, fns = {}
	, fnRe = /(['\/"])(?:\\?.)*?\1|\b(?:false|in|new|null|this|true|void)\b|\.\w+|\w+:/g



	// Function extensions
	// -------------------


	function construct(fn, args, len) {
		// bind version have bad performance and memory consumption
		// return new(F.bind.apply(this, A.concat.apply([null], args)))
		len = args.length
		return len ?
		(fns[len] || (fns[len] = Fn("t a->new t(a[" + Object.keys(args).join("],a[") + "])")))(fn, args) :
		new fn
	}

	F.partial = function() {
		var fn = this
		, args = slice(arguments)
		return function() {
			return fn.apply(this, args.concat.apply(args, arguments))
		}
	}

	// Run function once and return cached value or cached instance
	F.cache = function(instance, keyFn, cache) {
		var fn = wrapper.origin = this
		, c = wrapper.cached = cache || {}

		function wrapper() {
			var args = arguments
			, i = !!instance || this instanceof wrapper
			, k = keyFn ? keyFn.apply(fn, args) : i + ":" + args.length + ":" + slice(args)

			return k in c ? c[k] : (c[k] = i ? construct(fn, args) : fn.apply(this, args))
		}

		wrapper.extend = function() {
			return fn.extend.apply(fn, arguments).cache(instance, keyFn, cache)
		}
		wrapper[P] = fn[P] // prototype for better access on extending
		return wrapper
	}

	F.extend = function() {
		var arg
		, fn = this
		, i = 0

		function wrapper() {
			return fn.apply(this, arguments)
		}

		for (wrapper[P] = Object.create(fn[P]); arg = arguments[i++];) Object.merge(wrapper[P], arg)
		wrapper[P].constructor = wrapper
		return wrapper
	}


	// Non-standard
	Object.each = function(obj, fn, scope, key) {
		if (obj) for (key in obj) {
			hasOwn.call(obj, key) && fn.call(scope, obj[key], key, obj)
		}
	}

	// Object.assign ( target, source ) in ECMAScript 6
	Object.merge = function(target, source) {
		for (var key, i = 1; source = arguments[i++]; ) {
			for (key in source) if (hasOwn.call(source, key)) {
				target[key] = source[key]
			}
		}
		return target
	}

	Object.values = function(obj) {
		return Object.keys(obj || {}).map(function(e) {
			return obj[e]
		})
	}

	// Note: use for Object literals only,
	// as it returns false for custom objects,
	// like new Date or new YourObject.

	function isObject(obj) {
		return obj && obj.constructor === Object
	}

	function clone(source, temp, key) {
		if (isObject(source)) {
			temp = {}
			for (key in source) if (hasOwn.call(source, key)) {
				temp[key] = clone(source[key])
			}
			source = temp
		}
		return source
	}
	Object.clone = clone

	// Non-standard
	// IE<9 bug: [1,2].splice(0).join("") == "" but should be "12"
	A.remove = arrayRemove
	function arrayRemove() {
		var arr = this
		, len = arr.length
		, o = slice(arguments)
		, lastId = -1

		for (; len--; ) if (~o.indexOf(arr[len])) {
			arr.splice(lastId = len, 1)
		}
		return lastId
	}

	A.each = A.forEach
	A.fold = A.reduce
	A.foldr = A.reduceRight
	// uniq
	// first item preserved
	A.uniq = A.filter.partial(function(elem, i, arr) {
		return i == arr.indexOf(elem)
	})

	A.pushUniq = function(item) {
		return this.indexOf(item) == -1 ? this.push(item) : false
	}

	F.fn = function() {
		return this
	}

	S.fn = function(scope) {
		return Fn(this, scope)
	}

	Boolean.prototype.fn = function() {
		return this.valueOf() ? True : False
	}

	// THANKS: Oliver Steele http://www.osteele.com/sources/javascript/functional/
	function Fn(expr /*, scope, mask1, ..maskN */) {
		var args = []
		, arr = expr.match(/[^"']+?->|.+$/g)
		, scope = slice(arguments, 1)
		, key = scope.length + ":" + expr
		, fn = fns[key]

		if (!fn) {
			fn = expr.replace(fnRe, "").match(/\b[a-z_$][\w$]*/ig) || []
			for (; arr.length > 1; ) {
				expr = arr.pop()
				args = arr.pop().match(/\w+/g) || []
				arrayRemove.apply(fn, args)
				if (arr.length) {
					arr.push("function(" + args + "){return(" + expr + ")}" + (scope[0] ? ".bind(this)" : ""))
				}
			}
			expr = "return(" + expr + ")"

			if (scope[1]) {
				arr = Object.keys(scope.slice(1)).map(Fn("a->'__'+a"))
				args.unshift.apply(args, arr)
				expr = "with(" + arr.join(")with(") + "){" + expr + "}"
			}

			if (scope[0]) {
				expr = "with(this){" + expr + "}"
				if (fn[0]) expr = "var " + fn.uniq().join("='',") + "='';" + expr
			}

			fn = fns[key] = Function(args, expr)
		}

		return scope.length ? fn.bind.apply(fn, scope) : fn
	}

	exports.Fn = Fn



	function wait(fn) {
		var pending = 1
		function resume() {
			if (!--pending && fn) fn.call(this)
		}
		resume.wait = function() {
			pending++
			return resume
		}
		return resume
	}
	exports.Fn.wait = wait



	function hold(ignore) {
		var k
		, obj = this
		, hooks = []
		, hooked = []
		, _resume = wait(resume)
		ignore = ignore || obj.syncMethods || []

		for (k in obj) if (typeof obj[k] == "function" && ignore.indexOf(k) == -1) !function(k) {
			hooked.push(k, hasOwn.call(obj, k) && obj[k])
			obj[k] = function() {
				hooks.push(k, arguments)
				return obj
			}
		}(k)

		/**
		 * `wait` is already in hooked array,
		 * so override hooked method
		 * that will be cleared on resume.
		 */
		obj.wait = _resume.wait

		return _resume

		function resume() {
			for (var v, scope = obj, i = hooked.length; i--; i--) {
				if (hooked[i]) obj[hooked[i-1]] = hooked[i]
				else delete obj[hooked[i-1]]
			}
			// i == -1 from previous loop
			for (; v = hooks[++i]; ) {
				scope = scope[v].apply(scope, hooks[++i]) || scope
			}
			hooks = hooked = null
		}
	}
	exports.Fn.hold = hold

	function True() {
		return true
	}

	function False() {
		return false
	}

}(this, Object)







/*
* @version  0.6.0
* @author   Lauri Rooden <lauri@rooden.ee>
* @license  MIT License
*/



!function(Date, proto) {
	var maskRe = /("|')((?:\\?.)*?)\1|([YMD])\3\3\3?|([YMDHhmsWSZ])(\4?)|[uUASwoQ]/g
	, dateRe = /(\d+)[-.\/](\d+)[-.\/](\d+)/
	, timeRe = /(\d+):(\d+)(?::(\d+))?(\.\d+)?(?:\s*(?:(a)|(p))\.?m\.?)?(\s*(?:Z|GMT|UTC)?(?:([-+]\d\d):?(\d\d)?)?)?/i
	, unescapeRe = /\\(.)/g
	, map = { D:"Date", h:"Hours", m:"Minutes", s:"Seconds", S:"Milliseconds" }
	//, isoDateRe = /(\d{4})[-.\/]W(\d\d?)[-.\/](\d)/


	// ISO 8601 specifies numeric representations of date and time.
	//
	// The international standard date notation is
	// YYYY-MM-DD
	//
	// The international standard notation for the time of day is
	// hh:mm:ss
	//
	// Time zone
	//
	// The strings +hh:mm, +hhmm, or +hh (ahead of UTC)
	// -hh:mm, -hhmm, or -hh (time zones west of the zero meridian, which are behind UTC)
	//
	// 12:00Z = 13:00+01:00 = 0700-0500

	Date[proto].date = Date[proto].format = function(mask, _zone) {
		mask = Date.masks[mask] || mask || Date.masks["default"]

		var undef, zonediff
		, date = this
		, origin = +date
		, get = "get" + (mask.slice(0, 4) == "UTC:" ? (mask = mask.slice(4), "UTC") : "")
		, zone = _zone == undef ? date._z : _zone

		if (zone != undef && get == "get") {
			get = "getUTC"
			date.setTime( origin + (36e5 * zone) )
			zonediff = 60 * zone
		}

		mask = mask.replace(maskRe, function(match, quote, text, MD, single, pad) {
			text = MD == "Y"  ? date[get + "FullYear"]()
			: MD              ? Date.names[ date[get + (MD == "M" ? "Month" : "Day" ) ]() + ( match == "DDD" ? 24 : MD == "D" ? 31 : match == "MMM" ? 0 : 12 ) ]
			: single == "Y"   ? date[get + "FullYear"]() % 100
			: single == "W"   ? ( quote = new Date(origin + ((4 - (date[get + "Day"]()||7)) * 86400000))
			                    , Math.ceil(((quote.getTime() - quote["s" + get.slice(1) + "Month"](0, 1)) / 86400000 + 1 ) / 7)
					    )
			: single == "M"   ? date[get + "Month"]() + 1
			: single == "H"   ? date[get + "Hours"]() % 12 || 12
			: single == "Z"   ? ( quote = zonediff || get == "get" && -date.getTimezoneOffset() || 0
				, quote ? (quote < 0 ? ((quote=-quote), "-") : "+") + (quote < 600 ? "0" : "") + (0|(quote/60)) + ((quote%=60) ? (pad ? "" : ":") + quote : "") : "Z"
			)
			: single          ? date[get + map[single]]()
			: match == "u"    ? (date/1000)>>>0
			: match == "U"    ? origin
			: match == "Q"    ? ((date[get + "Month"]()/3)|0) + 1
			: match == "A"    ? Date[date[get + "Hours"]() > 11 ? "pm" : "am"]
			: match == "w"    ? date[get + "Day"]() || 7
			: match == "o"    ? new Date(origin + ((4 - (date[get + "Day"]()||7)) * 86400000))[get + "FullYear"]()
			: quote           ? text.replace(unescapeRe, "$1")
			: match
			if (match == "SS" && text < 100) text = "0" + text
			return pad && text < 10 && single != "Z" ? "0" + text : text
		})
		if (zonediff != undef) date.setTime( origin )
		return mask
	}

	Date[proto].tz = function(zone) {
		this._z = zone
		return this
	}

	Date.am = "AM"
	Date.pm = "PM"

	Date.masks = {
		"default": "DDD MMM DD YYYY hh:mm:ss",
		"iso": "UTC:YYYY-MM-DD'T'hh:mm:ss'Z'"
	}
	Date.names = "JanFebMarAprMayJunJulAugSepOctNovDecJanuaryFebruaryMarchAprilMayJuneJulyAugustSeptemberOctoberNovemberDecemberSunMonTueWedThuFriSatSundayMondayTuesdayWednesdayThursdayFridaySaturday".match(/.[a-z]+/g)

	//*/


	/*
	 * // In Chrome Date.parse("01.02.2001") is Jan
	 * num = +date || Date.parse(date) || ""+date;
	 */

	String[proto].date = Number[proto].date = function(format, zoneOut, zoneIn) {
		var undef, match, year, month
		, date = new Date()
		, num = +this || "" + this

		if (isNaN(num)) {
			if (match = num.match(dateRe)) {
				// Big endian date, starting with the year, eg. 2011-01-31
				// Middle endian date, starting with the month, eg. 01/31/2011
				// Little endian date, starting with the day, eg. 31.01.2011
				year = match[1] > 99 ? 1 : 3
				month = Date.middleEndian ? 4 - year : 2
				date.setFullYear(match[year], match[month] - 1, match[6 - month - year])
			}

			// Time
			match = num.match(timeRe) || [0, 0, 0]
			date.setHours(
				match[6] && match[1] < 12 ? +match[1] + 12 :
				match[5] && match[1] == 12 ? 0 : match[1],
				match[2], match[3]|0, (1000 * match[4])|0
			)

			// Timezone
			if (match[7]) {
				zoneIn = (match[8]|0) + ((match[9]|0)/(match[8]<0?-60:60))
			}
		} else date.setTime( num < 4294967296 ? num * 1000 : num )

		if (zoneIn != undef) date.setTime(date - (60 * zoneIn + date.getTimezoneOffset()) * 60000)

		return format ? date.format(format, zoneOut) : date
	}

}(Date, "prototype")








/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



!function(exports, Object) {
	var hasOwn = Object.prototype.hasOwnProperty

	exports.pointer = pointer
	exports.mergePatch = mergePatch

	function isObject(obj) {
		return !!obj && obj.constructor == Object
	}

	/**
	 * JSON Pointer
	 * @see http://tools.ietf.org/html/rfc6901
	 */

	function pointer(obj, path, value) {
		if (path) {
			path = path.split("/")
			for (
				var _key
				, _set = arguments.length > 2
				, i = 1
				, len = path.length
				; obj && i < len
				; ) {
				_key = path[i++].replace(/~1/g, "/").replace(/~0/g, "~")
				if (_set) {
					if (i == len) {
						// Reuse _set to keep existing value
						_set = obj[_key]
						obj[_key] = value
						return _set
					}
					if (!obj[_key] || typeof obj[_key] != "object") obj[_key] = {}
				}
				obj = obj[_key]
			}
		}
		return obj
	}

	/**
	 * JSON Merge Patch
	 * @see http://tools.ietf.org/html/rfc7396
	 */

	function mergePatch(target, patch, changed, _path, _key, _val, _nextPath, _undef, _len) {
		if (!_path) _path = ""

		if (isObject(patch)) {
			if (!isObject(target)) {
				target = {}
			}
			for (_key in patch) if (
				_undef !== (_val = patch[_key]) &&
				hasOwn.call(patch, _key) &&
				(
					_undef == _val ?
					_undef !== target[_key] && delete target[_key] :
					target[_key] !== _val
				)
			) {
				_nextPath = _path + "/" + _key.replace(/~/g, "~0").replace(/\//g, "~1")
				_len = changed && isObject(target[_key]) && changed.length
				if (_undef != _val) {
					target[_key] = mergePatch(target[_key], _val, changed, _nextPath)
				}
				if (_len === false || _len != changed.length) changed.push(_nextPath)
			}
		} else {
			target = patch
		}
		return target
	}

}(JSON, Object)






/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



!function(window, document) {
	// The addEventListener is supported in Internet Explorer from version 9.
	// https://developer.mozilla.org/en-US/docs/Web/Reference/Events/wheel
	// - IE8 always prevents the default of the mousewheel event.

	var Event = window.Event || (window.Event = {})
	, wheelDiff = 120
	, addEv = "addEventListener"
	, remEv = "removeEventListener"
	, prefix = window[addEv] ? "" : (addEv = "attachEvent", remEv = "detachEvent", "on")
	, fixEv = Event.fixEv = {
		wheel:
			"onwheel" in document      ? "wheel" :      // Modern browsers
			"onmousewheel" in document ? "mousewheel" : // Webkit and IE
			"DOMMouseScroll"                            // older Firefox
	}
	, fixFn = Event.fixFn = {
		wheel: function(el, _fn) {
			return function(e) {
				var delta = (e.wheelDelta || -e.detail || -e.deltaY)/wheelDiff
				if (delta) {
					if (delta < 1 && delta > -1) {
						var diff = (delta < 0 ? -1 : 1)/delta
						delta *= diff
						wheelDiff /= diff
					}
					//TODO: fix event
					// e.deltaY =
					// e.deltaX = - 1/40 * e.wheelDeltaX|0
					// e.target = e.target || e.srcElement
					_fn.call(el, e, delta)
				}
			}
		}
	}

	Event.Emitter = {
		on: on,
		non: non,
		off: non,
		once: once,
		one: once,
		emit: emit
	}

	function on(type, fn, scope, _origin) {
		var emitter = this
		, events = emitter._e || (emitter._e = {})
		;(events[type] || (events[type] = [])).unshift(scope, _origin, fn)
		return emitter
	}

	function non(type, fn, scope) {
		var i
		, emitter = this
		, events = emitter._e && emitter._e[type]
		if (events) {
			if (fn) for (i = events.length; i--; i--) {
				if ((events[i--] === fn || events[i] === fn) && events[i - 1] == scope) {
					events.splice(i - 1, 3)
					break
				}
			}
			else events.length = 0
		}
		return emitter
	}

	function once(type, fn, scope) {
		var emitter = this
		function remove() {
			emitter.non(type, fn, scope).non(type, remove, scope)
		}
		return emitter.on(type, remove, scope).on(type, fn, scope)
	}

	function emit(type) {
		var args, i
		, emitter = this
		if (type = (emitter._e && emitter._e[type])) {
			type = type.slice()
			for (i = type.length, args = type.slice.call(arguments, 1); i--; ) {
				type[i--].apply(type[--i] || emitter, args)
			}
		}
		return emitter
	}

	Event.add = function(el, ev, _fn) {
		var fn = fixFn[ev] && fixFn[ev](el, _fn) || _fn
		, fix = prefix ? function() {
			var e = window.event
			if (e) {
				e.target = e.srcElement
				e.preventDefault = preventDefault
				e.stopPropagation = stopPropagation
			}
			fn.call(el, e)
		} : fn

		el[addEv](prefix + (fixEv[ev] || ev), fix, false)

		on.call(el, ev, fix, el, _fn)
	}

	Event.remove = function(el, ev, fn) {
		var evs = el._e && el._e[ev]
		, id = evs && evs.indexOf(fn)
		if (id) {
			el[remEv](prefix + (fixEv[ev] || ev), evs[id + 1])
			evs.splice(id - 1, 3)
		}
	}

	function preventDefault() {
		this.returnValue = false
	}
	function stopPropagation() {
		this.cancelBubble = this.cancel = true
	}

	Event.stop = function(e) {
		if (e.preventDefault) {
			e.stopPropagation()
			e.preventDefault()
		}
		return false
	}

	Event.removeAll = function(el, ev, key) {
		if (el._e) for (key in el._e) {
			if (!ev || key === ev) Event.remove(el, key)
		}
	}

}(this, document)







/*
* @version  0.6.0
* @author   Lauri Rooden <lauri@rooden.ee>
* @license  MIT License
*/




/*
* Windows versions of Opera have a bit of buggy behavior:
* when you type the +, -, *, or / keys on the keypad,
* then two keypress events are triggered instead of one.
* This has been observed on Opera 11 and Opera 8.5.
* http://unixpapa.com/js/key.html
*/

//** Event.keymap
!function(Event) {
	var maps = []
	, keys = {
		8:"backspace", 9:"tab",
		13:"enter", 16:"shift", 17:"ctrl", 18:"alt", 19:"pause",
		20:"caps", 27:"esc",
		33:"pgup", 34:"pgdown",
		35:"end", 36:"home",
		37:"left", 38:"up", 39:"right", 40:"down",
		45:"ins", 46:"del",
		112:"f1", 113:"f2", 114:"f3", 115:"f4", 116:"f5", 117:"f6", 118:"f7", 119:"f8",
		120:"f9", 121:"f10", 122:"f11", 123:"f12"
	}
	//, is_down = {}

	function _key(e, code, chr) {
		var fn, map
		, i = 0
		, el = e.target || e.srcElement
		, input = /INPUT|TEXTAREA|SELECT/i.test((el.nodeType == 3 ? el.parentNode : el).tagName)

		while (map = maps[i++]) {
			if (!input || map.enable_input) {
				fn = map[code] ||
				map[chr] ||
				map.num && code > 47 && code < 58 && (chr|=0, map.num) ||
				map.all
			}
			if (fn || !map.bubble) break
		}
		if (fn) fn(e, chr, el)
	}

	Event.setKeyMap = maps.unshift.bind(maps)

	Event.rmKeyMap = function(map) {
		var i = maps.indexOf(map||maps[0])
		if (i > -1) maps.splice(i, 1)
	}

	function keydown(e) {
		var code = e.keyCode || e.which
		// Otherwise IE backspace navigates back
		if (code == 8 && maps[0] && maps[0].backspace) {
			Event.stop(e)
		}
	//	is_down[ code ] = 1
	}
	Event.add(document, "keydown", keydown)

	function keyup(e) {
		var code = e.keyCode || e.which
		if (code > 95 && code < 106) code -= 48
		var key = keys[code] || String.fromCharCode(code) || code
		_key(e, code, key)
		if (e.ctrlKey && code != 17) _key(e, code, "ctrl+" + key)
		if (e.altKey && code != 18) _key(e, code, "alt+" + key)
		//e.shiftKey
		//is_down[ key ] = 0
		//is_down.ctrl && _key(e, code, "ctrl+"+key, el, input)
	}
	Event.add(document, "keyup", keyup)
}(Event)

//*/







/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



// https://github.com/browserstate/history.js/wiki/The-State-of-the-HTML5-History-API

!function(window, document, history) {
	var cb, base, last_route, iframe, tick, last
	, loc = location
	, clean_route = /^[#\/\!]+|[\s\/]+$/g

	// The JScript engine used in IE doesn't recognize vertical tabulation character
	// http://webreflection.blogspot.com/2009/01/32-bytes-to-know-if-your-browser-is-ie.html
	// oldIE = "\v" == "v"
	//
	// The documentMode is an IE only property, supported in IE8+.
	//
	// Starting in Internet Explorer 9 standards mode, Internet Explorer 10 standards mode,
	// and win8_appname_long apps, you cannot identify the browser as Internet Explorer
	// by testing for the equivalence of the vertical tab (\v) and the "v".
	// In earlier versions, the expression "\v" === "v" returns true.
	// In Internet Explorer 9 standards mode, Internet Explorer 10 standards mode,
	// and win8_appname_long apps, the expression returns false.
	, ie6_7 = !+"\v1" && (document.documentMode||1) < 8

	function getUrl(_loc) {
		var url
		/*** PUSH
		if (base) {
			url = loc.pathname.slice(base.length)
		} else {
		//*/
			// bug in Firefox where location.hash is decoded
			// bug in Safari where location.pathname is decoded

			// var hash = loc.href.split('#')[1] || '';
			// https://bugs.webkit.org/show_bug.cgi?id=30225
			// https://github.com/documentcloud/backbone/pull/967
			url = (_loc || loc).href.split("#")[1] || ""
		/*** PUSH
		}
		//*/
		// Fix decode
		// http://unixpapa.com/js/querystring.html
		url = decodeURIComponent(url.replace(/\+/g, " "))
		return url.replace(clean_route, "")
	}

	function setUrl(url, replace) {
		url = encodeURIComponent(url).replace(/%20/g, "+").replace(/%2F/gi, "/")
		/*** PUSH
		if (base) {
			history[replace ? "replaceState" : "pushState"](null, null, base + url)
		} else {
		//*/
			loc[replace ? "replace" : "assign"]("#" + url)
			// Opening and closing the iframe tricks IE7 and earlier
			// to push a history entry on hash-tag change.
			if (iframe && getUrl() !== getUrl(iframe.location) ) {
				iframe.location[replace ? "replace" : iframe.document.open().close(), "assign"]("#" + url)
			}
		/*** PUSH
		}
		//*/
		checkUrl()
	}

	function checkUrl() {
		if (last_route != (last_route = getUrl()) && cb) cb(last_route)
	}

	history.getUrl = getUrl
	history.setUrl = setUrl

	history.start = function(_cb, _base) {
		if (!cb) {
			/*** PUSH
			if (_base && history.pushState) {
				base = _base
				// Chrome and Safari emit a popstate event on page load, Firefox doesn't.
				// Firing popstate after onload is as designed.
				//
				// See the discussion on https://bugs.webkit.org/show_bug.cgi?id=41372,
				// https://code.google.com/p/chromium/issues/detail?id=63040
				// and the change to the HTML5 spec that was made:
				// http://html5.org/tools/web-apps-tracker?from=5345&to=5346.
				window.onpopstate = checkUrl
			} else
			//*/
				if ("onhashchange" in window && !ie6_7) {
				// There are onhashchange in IE7 but its not get emitted
				//
				// Basic support:
				// Chrome 5.0, Firefox (Gecko) 3.6 (1.9.2), IE 8.0, Opera 10.6, Safari 5.0
				window.onhashchange = checkUrl
			} else {
				if (ie6_7) {
					// IE<9 encounters the Mixed Content warning when the URI javascript: is used.
					// IE5/6 additionally encounters the Mixed Content warning when the URI about:blank is used.
					// src="//:"
					iframe = document.body.appendChild(document.createElement('<iframe class="hide" tabindex="-1">')).contentWindow
				}
				tick = setInterval(function(){
					var cur = getUrl()
					if (iframe && last === cur) cur = getUrl(iframe.location)
					if (last !== cur) {
						last = cur
						iframe ? setUrl(cur) : checkUrl()
					}
				}, 60)
			}
		}
		cb = _cb
		checkUrl()
	}
}(this, document, history)







/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 * @homepage   https://www.litejs.com/elements-lite/
 */



!function(window, document, protoStr) {
	var currentLang, styleNode
	// JScript engine in IE<9 does not recognize vertical tabulation character
	, ie678 = !+"\v1"
	, ie67 = ie678 && (document.documentMode|0) < 8
	, hasOwn = Object[protoStr].hasOwnProperty
	, wrapProto = []
	, body = document.body
	, createElement = document.createElement
	, txtAttr = "textContent" in body ? "textContent" : "innerText"
	, elCache = El.cache = {}
	, scopeSeq = 0
	, scopeData = El.data = { _: i18n }
	, proto = (window.HTMLElement || window.Element || El)[protoStr]
	, templateRe = /^([ \t]*)(@?)((?:("|')(?:\\?.)*?\4|[-\w\:.#\[\]=])*)[ \t]*(.*?)$/gm
	, renderRe = /[;\s]*(\w+)(?:\s*\:((?:(["'\/])(?:\\?.)*?\3|[^;])*))?/g
	, bindings = El.bindings = {
		"class": function(name, fn) {
			toggleClass.call(this, name, arguments.length < 2 || fn)
		},
		css: function(key, val) {
			this.style[key.camelCase()] = val || ""
		},
		data: function(key, val) {
			this.attr("data-" + key, val)
		},
		html: function(html) {
			this.innerHTML = html
		}
	}

	// Element.matches is supported from Chrome 34, Firefox 34
	, matches = proto.matches = proto.matches || function(selector) {
		return !!selectorFn(selector)(this)
	}
	// Element.closest is supported from Chrome 41, Firefox 35
	, closest = proto.closest = proto.closest || function(selector) {
		for (var el = this; el; el = el.parentNode) {
			if (matches.call(el, selector)) return el
		}
		return null
	}

	, selectorRe = /([.#:[])([-\w]+)(?:\((.+?)\)|([~^$*|]?)=(("|')(?:\\?.)*?\6|[-\w]+))?]?/g
	, selectorLastRe = /([\s>+]*)(?:("|')(?:\\?.)*?\2|\(.+?\)|[^\s+>])+$/
	, selectorSplitRe = /\s*,\s*(?=(?:[^'"()]|"(?:\\?.)*?"|'(?:\\?.)*?'|\(.+?\))+$)/
	, selectorCache = {}
	, selectorMap = {
		"first-child": "(a=_.parentNode)&&a.firstChild==_",
		"last-child": "(a=_.parentNode)&&a.lastChild==_",
		".": "~_.className.split(/\\s+/).indexOf(a)",
		"#": "_.id==a",
		"^": "!a.indexOf(v)",
		"|": "a.split('-')[0]==v",
		"$": "a.slice(-v.length)==v",
		"~": "~a.split(/\\s+/).indexOf(v)",
		"*": "~a.indexOf(v)"
	}

	function findEl(node, sel, first) {
		var el
		, i = 0
		, out = []
		, next = node.firstChild
		, fn = selectorFn(sel)

		for (; (el = next); ) {
			if (fn(el)) {
				if (first) return el
				out.push(el)
			}
			next = el.firstChild || el.nextSibling
			while (!next && ((el = el.parentNode) !== node)) next = el.nextSibling
		}
		return first ? null : out
	}

	function selectorFn(str) {
		// jshint evil:true
		return selectorCache[str] ||
		(selectorCache[str] = Function("_,v,a,b", "return " +
			str.split(selectorSplitRe).map(function(sel) {
				var relation, from
				, rules = ["_&&_.nodeType==1"]
				, parentSel = sel.replace(selectorLastRe, function(_, _rel, a, start) {
					from = start + _rel.length
					relation = _rel.trim()
					return ""
				})
				, tag = sel.slice(from).replace(selectorRe, function(_, op, key, subSel, fn, val, quotation) {
					rules.push(
						"((v='" +
						(subSel || (quotation ? val.slice(1, -1) : val) || "").replace(/'/g, "\\'") +
						"'),(a='" + key + "'),1)"
						,
						selectorMap[op == ":" ? key : op] ||
						"(a=_.getAttribute(a))" +
						(fn ? "&&" + selectorMap[fn] : val ? "==v" : "")
					)
					return ""
				})

				if (tag && tag != "*") rules[0] += "&&_.nodeName=='" + tag.toUpperCase() + "'"
				if (parentSel) rules.push(
					relation == "+" ? "(a=_.previousSibling)" : "(a=_.parentNode)",
					( relation ? "a.matches&&a.matches('" : "a.closest&&a.closest('" ) + parentSel + "')"
				)
				return rules.join("&&")
			}).join("||")
		))
	}


	/**
	 * Turns CSS selector like syntax to DOM Node
	 * @returns {Node}
	 *
	 * @example
	 * El("input#12.nice[type=checkbox]:checked:disabled[data-lang=en].class")
	 * <input id="12" class="nice class" type="checkbox" checked="checked" disabled="disabled" data-lang="en">
	 */

	function El(name, args, silence) {
		var el
		, pre = {}
		name = name.replace(selectorRe, function(_, op, key, _sub, fn, val, quotation) {
			val = quotation ? val.slice(1, -1) : val || key
			pre[op =
				op == "." ?
				(fn = "~", "class") :
				op == "#" ?
				"id" :
				key
			] = fn && pre[op] ?
				fn == "^" ? val + pre[op] :
				pre[op] + (fn == "~" ? " " : "") + val :
				val
			return ""
		}) || "div"

		// NOTE: IE-s cloneNode consolidates the two text nodes together as one
		// http://brooknovak.wordpress.com/2009/08/23/ies-clonenode-doesnt-actually-clone/
		el = (elCache[name] || (elCache[name] = document.createElement(name))).cloneNode(true).attr(pre)

		return silence || !args ? el :
		(typeof args == "object" ? attr : append).call(el, args)
	}
	window.El = El

	function append(child, before) {
		var fragment
		, el = this
		, i = 0
		, tmp = typeof child
		if (child) {
			if (tmp == "string" || tmp == "number") child = document.createTextNode(child)
			else if ( !("nodeType" in child) && "length" in child ) {
				// document.createDocumentFragment is unsupported in IE5.5
				// fragment = "createDocumentFragment" in document ? document.createDocumentFragment() : El("div")
				for (
					tmp = child.length
					, fragment = document.createDocumentFragment();
					i < tmp; ) append.call(fragment, child[i++])
				child = fragment
			}

			if (child.nodeType) {
				tmp = el.insertBefore ? el : el[el.length - 1]
				tmp.insertBefore(child,
					(before === true ? tmp.firstChild :
					typeof before == "number" ? tmp.childNodes[
						before < 0 ? tmp.childNodes.length - before - 2 : before
					] : before) || null
				)
			}
		}
		return el
	}
	proto.append = append

	proto.after = function(silbing, before) {
		// call append so it works with DocumentFragment
		append.call(silbing.parentNode, this, before ? silbing : silbing.nextSibling)
		return this
	}

	proto.to = function(parent, before) {
		append.call(parent, this, before)
		return this
	}

	function hasClass(name) {
		return this.className.split(/\s+/).indexOf(name) > -1
	}
	proto.hasClass = hasClass

	function addClass(name) {
		var el = this
		, current = el.className

		if (current) {
			name = hasClass.call(el, name) ? current : current + " " + name
		}

		if (current != name) el.className = name
		return el
	}
	proto.addClass = addClass

	function rmClass(name) {
		var el = this

		if (hasClass.call(el, name)) {
			el.className = (" " + el.className + " ").replace(" " + name + " ", " ").trim()
		}

		return el
	}
	proto.rmClass = rmClass

	function toggleClass(name, force) {
		if (arguments.length == 1) force = !hasClass.call(this, name)
		return ( force ? addClass : rmClass ).call(this, name), force
	}
	proto.toggleClass = toggleClass

	proto.empty = function() {
		for (var node, el = this; node = el.firstChild; ) kill.call(node)
		return el
	}

	function kill() {
		var id
		, el = this
		if (el.emit) el.emit("kill")
		if (el.parentNode) el.parentNode.removeChild(el)
		if (Event.removeAll) Event.removeAll(el)
		if (el.empty) el.empty()
		if (id = el.attr && el.attr("data-scope")) {
			delete elScope[id]
		}
		if (el.valObject) el.valObject = null
		return el
	}
	proto.kill = kill

	proto.on = function(ev, fn) {
		// element.setCapture(retargetToElement)
		Event.add(this, ev, fn)
		return this
	}

	proto.non = function(ev, fn) {
		Event.remove(this, ev, fn)
		return this
	}
	proto.emit = function() {
		Event.Emitter.emit.apply(this, arguments)
	}

	proto.attr = attr
	function attr(key, val) {
		var current
		, el = this

		if (key && key.constructor == Object) {
			for (val in key) {
				attr.call(el, val, key[val])
			}
			return el
		}

		current = el.getAttribute(key)

		if (arguments.length == 1) {
			return current
		}
		// Note: IE5-7 doesn't set styles and removes events when you try to set them.
		//
		// in IE6, a label with a for attribute linked to a select list
		// will cause a re-selection of the first option instead of just giving focus.
		// http://webbugtrack.blogspot.com/2007/09/bug-116-for-attribute-woes-in-ie6.html

		// there are bug in IE<9 where changed 'name' param not accepted on form submit
		// IE8 and below support document.createElement('<P>')
		//
		// http://www.matts411.com/post/setting_the_name_attribute_in_ie_dom/
		// http://msdn.microsoft.com/en-us/library/ms536614(VS.85).aspx

		//** modernBrowser
		if (ie67 && (key == "id" || key == "name" || key == "checked")) {
			el.mergeAttributes(createElement('<INPUT '+key+'="' + val + '">'), false)
		} else
		//*/
		if (key == "class") {
			addClass.call(el, val)
		} else if (val || val === 0) {
			if (current != val) {
				el.setAttribute(key, val)
			}
		} else if (current) {
			el.removeAttribute(key)
		}
	}

	function elScope(node, parent, _scope) {
		if (_scope = elScope[attr.call(node, "data-scope")]) {
			return _scope
		}
		if (!parent || parent === true) {
			_scope = closest.call(node, "[data-scope]")
			_scope = _scope && elScope[attr.call(_scope, "data-scope")] || scopeData
		}
		if (parent) {
			attr.call(node, "data-scope", ++scopeSeq)
			_scope = elScope[scopeSeq] = Object.create(parent = (_scope || parent))
			_scope._super = parent
		}
		return _scope
	}
	El.scope = elScope

	function render(scope, skipSelf) {
		var bind, newBind, fn
		, node = this

		if (node.nodeType != 1) {
			return node
		}

		// TODO:2015-05-25:lauri:Use elScope
		scope = elScope[attr.call(node, "data-scope")]
		|| scope
		|| (bind = closest.call(node, "[data-scope]")) && elScope[attr.call(bind, "data-scope")]
		|| scopeData

		if (bind = !skipSelf && attr.call(node, "data-bind")) {
			newBind = bind
			// i18n(bind, lang).format(scope)
			// document.documentElement.lang
			// document.getElementsByTagName('html')[0].getAttribute('lang')

			fn = "data b r->data&&(" + bind.replace(renderRe, function(match, name, args) {
				return bindings[name] ?
				(hasOwn.call(bindings[name], "once") && (newBind = newBind.replace(match, "")),
					"(r=b['" + name + "'].call(this" + (bindings[name].raw ? ",data,'" + args + "'" : args ? "," + args : "") + ")||r),") :
				"this.attr('" + name + "'," + args + "),"
			}) + "r)"
			if (bind != newBind) attr.call(node, "data-bind", newBind)

			try {
				if (Fn(fn, node, scope)(scope, bindings)) {
					return node
				}
			} catch (e) {
				//** debug
				e.message += " in binding: " + bind
				console.error(e)
				if (window.onerror) window.onerror(e.message, e.fileName, e.lineNumber)
				//*/
				return node
			}
		}

		for (bind = node.firstChild; bind; bind = newBind) {
			newBind = bind.nextSibling
			render.call(bind, scope)
		}
		//** modernBrowser
		if (ie678 && node.nodeName == "SELECT") {
			node.parentNode.insertBefore(node, node)
		}
		//*/
		return node
	}

	proto.render = render

	// In Safari 2.x, innerText functions properly only
	// if an element is neither hidden (via style.display == "none")
	// nor orphaned from the document.
	// Otherwise, innerText results in an empty string.
	//
	// textContent is suported from IE9
	// Opera 9-10 have Node.text so we use Node.txt

	proto.txt = bindings.txt = function(newText) {
		return arguments.length && this[txtAttr] != newText ? (
			//** modernBrowser
			// Fix for IE5-7
			//(ie67 && this.tagName == "OPTION" && (this.label = newText)),
			//*/
			this[txtAttr] = newText
		) : this[txtAttr]
	}

	proto.val = bindings.val = function(val) {
		var el = this
		, type = el.type
		, opts = el.options

		if (arguments.length) {
			return el.value = val
		}

		if (opts) {
			if (type == "select-multiple") {
				for (val = [], type = 0; el = opts[type++]; ) {
					if (el.selected && !el.disabled) {
						val.push(el.valObject || el.value)
					}
				}
				return val
			}
			// IE8 throws error when accessing to options[-1]
			val = el.selectedIndex
			el = val > -1 && opts[val] || el
		}

		return (type == "checkbox" || type == "radio") && !el.checked ? null :
		el.valObject || el.value
	}

	//** modernBrowser
	// Note: querySelector in IE8 supports only CSS 2.1 selectors
	proto.find = !ie678 && proto.querySelector || function(sel) {
		return findEl(this, sel, true)
	}

	proto.findAll = proto.querySelectorAll ?
		function(sel) {
			return new ElWrap(this.querySelectorAll(sel))
		} :
		function(sel) {
			return new ElWrap(findEl(this, sel))
		}
	/*/
	proto.find = proto.querySelector
	proto.findAll = function(sel) {
		return new ElWrap(this.querySelectorAll(sel))
	}
	//*/


	function ElWrap(nodes) {
		/**
		 *  1. Extended array size will not updated
		 *     when array elements set directly in Android 2.2.
		 */
		for (var pos = this.length /* 1 */ = nodes.length; pos--; ) {
			this[pos] = nodes[pos]
		}
	}
	El.wrap = ElWrap

	ElWrap[protoStr] = wrapProto

	Object.keys(proto).each(addWrapProto)

	function addWrapProto(key) {
		var first = key == "closest" || key == "find"

		wrapProto[key] = wrap
		function wrap() {
			for (var val, i = 0, len = this.length; i < len; ) {
				val = proto[key].apply(this[i++], arguments)
				if (first && val) return val
			}
			return first ? null : this
		}
	}

	addWrapProto("closest")

	wrapProto.cloneNode = function(deep) {
		return new ElWrap(this.map(function(el) {
			return el.cloneNode(deep)
		}))
	}

	//** modernBrowser
	// IE 6-7
	if (proto == El[protoStr]) {
		document.createElement = function(name) {
			return Object.merge(createElement(name), proto)
		}

		// NOTE: document.body will not get extended with later added extensions
		Object.merge(body, proto)
	}
	//*/

	El[protoStr] = proto

	El.get = function(id) {
		if (typeof id == "string") id = document.getElementById(id)
		//** modernBrowser
		return id && id.to ? id : Object.merge(id, proto)
		/*/
		return id
		//*/
	}

	El.css = function(str) {
		if (!styleNode) {
			// Safari and IE6-8 requires dynamically created
			// <style> elements to be inserted into the <head>
			styleNode = El("style").to(document.getElementsByTagName("head")[0])
		}
		if (styleNode.styleSheet) styleNode.styleSheet.cssText += str
		else styleNode.appendChild(document.createTextNode(str))
	}

	//** templates

	function tpl(str) {
		var root = document.createDocumentFragment()
		, parent = root
		, stack = [-1]

		function work(all, indent, plugin, name, q, text) {
			for (q = indent.length; q <= stack[0]; ) {
				stack.shift()
				parent = (parent.plugin) ? parent.plugin.done() : parent.parentNode || parent[0].parentNode
			}

			if (parent.txtMode) {
				parent.txt += all + "\n"
			} else if (plugin) {
				if (El.plugins[name]) {
					parent = (new El.plugins[name](parent, text)).el
					stack.unshift(q)
				} else {
					parent.append(all)
				}
			} else {
				if (name) {
					parent = El(name, 0, 1).to(parent)
					// TODO:2015-02-27:lauri:should we find a child to where put a content?
					stack.unshift(q)
				}
				if (text) {
					q = text.charAt(0)
					name = text.slice(1)
					if (q == ">") {
						(indent + " " + name).replace(templateRe, work)
					} else if (q == "|" || q == "\\") {
						parent.append(name) // + "\n")
					} else if (q != "/") {
						if (q != "&") {
							name = (parent.tagName == "INPUT" ? "val" : "txt")
							+ ":_('" + text.replace(/'/g, "\\'") + "').format(data)"
						}
						q = attr.call(parent, "data-bind")
						attr.call(parent, "data-bind", (q ? q + ";" : "") + name)
					}
				}
			}
		}
		str.replace(templateRe, work)
		root = root.childNodes
		return root.length == 1 ? root[0] : root
	}

	function template(parent, name) {
		var t = this
		t.name = name
		t.parent = parent
		t.el = El("div")
		t.el.plugin = t
	}

	template[protoStr] = {
		_done: function() {
			var t = this
			, el = t.el.childNodes.length > 1 ? new ElWrap(t.el.childNodes) : t.el.firstChild
			t.el.plugin = t.el = t.parent = null
			return el
		},
		done: function() {
			var t = this
			, parent = t.parent
			elCache[t.name] = t._done()
			return parent
		}
	}

	function js(parent) {
		var t = this
		t.txtMode = t.parent = parent
		t.txt = ""
		t.plugin = t.el = t
	}

	js[protoStr].done = Fn("Function(this.txt)(),this.parent")

	El.plugins = {
		binding: js.extend({
			done: function() {
				Object.merge(bindings, Function("return({" + this.txt + "})")())
				return this.parent
			}
		}),
		css: js.extend({
			done: Fn("El.css(this.txt),this.parent")
		}),
		js: js,
		template: template
	}

	El.view = El.tpl = tpl

	El.include = function(id, data, parent) {
		var src = El.get(id)
		new template(null, id).el.append( tpl(src.innerHTML) ).plugin.done()
		src.kill()
	}
	//*/


	//** i18n
	function i18n(text, lang) {
		lang = i18n[i18nGet(lang) || currentLang]
		return lang[text] ||
		lang[text = text.slice(text.indexOf(":") + 1) || text] ||
		text
	}
	El.i18n = i18n

	function i18nGet(lang) {
		return lang && (
			i18n[lang = ("" + lang).toLowerCase()] ||
			i18n[lang = lang.split("-")[0]]
		) && lang
	}

	function i18nUse(lang) {
		lang = i18nGet(lang)
		if (lang && currentLang != lang) {
			i18n[currentLang = i18n.current = lang] = i18n[currentLang] || {}
		}
		return currentLang
	}

	function i18nAdd(lang, texts) {
		if (i18n.list.indexOf(lang) == -1) i18n.list.push(lang)
		Object.merge(i18n[lang] || (i18n[lang] = {}), texts)
		if (!currentLang) i18nUse(lang)
	}

	i18n.list = []
	i18n.get = i18nGet
	i18n.use = i18nUse
	i18n.add = i18nAdd
	i18n.def = function(map) {
		Object.each(map, function(name, tag) {
			i18nAdd(tag, map)
		})
	}
	String[protoStr].lang = function(lang) {
		return i18n(this, lang)
	}
	// navigator.userLanguage for IE, navigator.language for others
	// var lang = navigator.language || navigator.userLanguage;
	// i18nUse("en")
	//*/

}(window, document, "prototype")






/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



!function(prototype) {
	var undef
	, A = Array[prototype]
	, N = Number[prototype]
	, S = String[prototype]
	, formatRe = /{(?!\\)((?:(["'/])(?:\\?.)*?\2|[^}])*)}/g
	, filterRe = /\|\s*(\w+)(?:\s*\:((?:(["'\/])(?:\\?.)*?\3|[^|])*))?/g
	, digitRe = /^\s*\d+/
	, numbersRe = /-?\d+\.?\d*/g
	, unescapeRe = /{\\/g

	S.format = function() {
		var args = A.slice.call(arguments)
		args.unshift(0)
		return this.replace(formatRe, function(_, arg) {
			args[0] = arg.replace(digitRe, "this[$&]").replace(filterRe, ".$1($2)")
			return Fn.apply(null, args)()
		}).replace(unescapeRe, "{")
	}

	N.format = function(data) {
		return "" + this
	}

	S.safe = function() {
		return this
		.replace(/&/g, "&amp;")
		.replace(/</g, "&lt;")
		.replace(/>/g, "&gt;")
		.replace(/\"/g, "&quot;")
	}

	S.capitalize = function() {
		return this.charAt(0).toUpperCase() + this.slice(1)
	}

	S.camelCase = function() {
		return this.replace(/[ _-]+([a-z])/g, function(_, a) {
			return a.toUpperCase()
		})
	}

	S.lower = S.toLowerCase
	S.upper = S.toUpperCase

	N.step = function(a, add) {
		var x = ("" + a).split(".")
		, steps = this / a
		, n = ~~(steps + (add == undef ? .5 : add === 1 && steps == (steps|0) ? 0 : +add)) * a
		return "" + (1 in x ? n.toFixed(x[1].length) : n)
	}

	S.step = function(a, add) {
		return this.replace(numbersRe, function(num) {
			return (+num).step(a, add)
		})
	}

	N.scale = function() {
		return words(this, [1000,1000,1000], ["","k","M","G"], {"default": "{0}{1}"})
	}

	S.scale = function() {
		return this.replace(numbersRe, function(num) {
			return (+num).scale()
		})
	}

	S.pick = N.pick = function() {
		var val = this + "="
		for (var s, a = arguments, i = 0, len = a.length; i < len;) {
			s = a[i++]
			if (s.indexOf(val) == 0) {
				s = s.slice(val.length)
				i = len
			}
		}
		return s.replace("#", this)
	}

	S.plural = N.plural = function() {
		// Plural-Forms: nplurals=2; plural=n != 1;
		// http://www.gnu.org/software/gettext/manual/html_mono/gettext.html#Plural-forms
		return arguments[ +Fn("n->" + (String.plural || "n!=1"))( parseFloat(this) ) ].replace("#", this)
	}

	A.pluck = function(name) {
		for (var arr = this, i = arr.length, out = []; i--; ) {
			out[i] = arr[i][name]
		}
		return out
	}

	function words(input, steps, units, strings, overflow) {
		var n = +input
		, i = 0
		, s = strings || {"default": "{0} {1}{2}"}

		while(n>=steps[i])n/=steps[i++]
		if (i == steps.length && overflow) return overflow(this)
		i=units[i]
		return (s[n<2?i:i+"s"]||s["default"]).format({0:n, 1:i, 2:n<2?"":"s"})
	}
	Number.words = words
}("prototype")






/*
 * @version    0.6.0
 * @date       2016-02-19
 * @stability  1 - Experimental
 * @author     Lauri Rooden <lauri@rooden.ee>
 * @license    MIT License
 */



!function(exports) {
	var fn, lastView, lastParams, lastUrl
	, fnStr = ""
	, reStr = ""
	, views = View.views = {}
	, groupsCount = 1
	, escapeRe = /[.*+?^=!:${}()|\[\]\/\\]/g
	, parseRe = /\{([\w%.]+?)\}|.[^{\\]*?/g

	function escapeRegExp(string) {
		return string.replace(escapeRe, "\\$&")
	}

	function View(route, el, parent, selector) {
		var view = this
		if (views[route]) {
			if (el) views[route].init(el, parent, selector)
			return views[route]
		}
		if (!(view instanceof View)) return new View(route, el, parent, selector)
		views[view.route = route] = view
		view.init(el, parent, selector)

		if (route.charAt(0) != "#") {
			var startLen = groupsCount++
			, params = ","
			, _re = route.replace(parseRe, function(_, key) {
				return key ?
					(params += "o['" + key + "']=u[" + (groupsCount++) + "],") && "([^/]+?)" :
					escapeRegExp(_)
			})

			fnStr += "u[" + startLen + "]?(" + params.slice(1) + "'" + route + "'):"
			reStr += (reStr ? "|" : "") + "(" + _re + ")"

			fn = new Function(
				"u,o,r",
				"return (u=/^\\/?(?:" + reStr + ")[\\/\\s]*$/.exec(u))?(" + fnStr + "r):r"
			)
		}
	}

	View.prototype = {
		init: function(el, parent, selector) {
			var view = this
			view.el = el
			view.parent = typeof parent == "string" ? View(parent) : parent
			view.selector = selector
		},
		show: function(params) {
			var child
			, view = lastView = this
			params = params || {}
			View.active = view.route
			if (view.open) {
				view.close()
			}
			for (; view; child = view, view = view.parent) {
				if (view.child && view.child != child) {
					view.child.close()
				}
				view.child = child
			}
			// child is now the root view
			child.ping(lastParams = params)
		},
		close: function() {
			var view = this

			if (view.open) {
				view.emit("close")
				if (view.child) view.child.close()
				view.open.kill()
				view.open = view.parent.child = null
			}
		},
		wait: function(params, emit) {
			var view = this
			, parent = view.parent
			params._p = 1 + (params._p | 0)
			return function() {
				if (--params._p || lastParams != params) return
				lastParams = Object.merge({}, params)
				if (view.el && view.parent == parent) {
					view.ping(lastParams, !emit)
				} else {
					;(view.el ? lastView : View("404")).show(lastParams)
				}
			}
		},
		ping: function(params, silent) {
			var view = this
			, parent = view.parent
			, child = view.child

			if (!view.el && view.file) {
				return xhr.load(
					view.file
					.replace(/^|,/g, "$&" + (View.base || ""))
					.split(","),
					view.wait(params, 1)
				)
			}

			if (!silent) {
				view.emit("ping", params)
				View.emit("ping", view.route, params)
			}

			if (lastParams == params && !params._p) {
				var type = typeof view.el
				if (type == "function") view.el = view.el()
				else if (type == "string") view.el = El.tpl(view.el)
				if (parent && !view.open) {
					view.open = view.el.cloneNode(true)
					parent.emit("beforeChild", params)
					view.open.to(
						parent.selector && parent.open && parent.open.find(parent.selector) || parent.open || parent.el
					)
					view.open.render()
				}
				if (child) {
					child.ping(params)
				}
			}

			if (lastParams == params && lastView == view && !params._p) {
				view.emit("show", params)
				View.emit("show", view.route, params)
			}
		}
	}

	Object.merge(View, Event.Emitter)
	Object.merge(View.prototype, Event.Emitter)

	View.home = "home"

	View.show = function(url) {
		if (url === true) {
			url = lastUrl
			lastUrl = 0
		}
		var params = {}
		, view = View(fn(url || View.home, params, "404"))
		if (!view.open || lastUrl != url) {
			params._u = lastUrl = url
			view.show(El.data.route = params)
		}
	}

	View.def = function(str) {
		for (var match, re = /(\S+) (\S+)/g; match = re.exec(str);) {
			match[2].split(",").map(function(route) {
				View(route).file = match[1]
			})
		}
	}

	exports.View = View

	El.plugins.view = El.plugins.template.extend({
		done: function() {
			var t = this
			, parent = t.parent
			, arr = t.name.split(/\s+/)
			View(arr[0], t._done(), arr[1], arr[2])
			return parent
		}
	})

	var dummy = El("div")

	El.plugins["view-link"] = El.plugins.template.extend({
		done: function() {
			var t = this
			, arr = t.name.split(/\s+/)
			View(arr[0], dummy, arr[2])
			.on("ping", function() {
				View.show(arr[1])
			})
			return t.parent
		}
	})
}(this)



!function(bindings) {
	var hasOwn = Object.prototype.hasOwnProperty
	, slice = Array.prototype.slice

	bindingFn.once =
	bindingIf.once =
	bindingWith.once =
	bindingOn.once =
	emitForm.once =
	bindingsEach.raw = bindingsEach.once =
	true

	bindings.fn = bindingFn
	function bindingFn(fn) {
		return fn.apply(this, slice.call(arguments, 1))
	}

	bindings["if"] = bindingIf
	function bindingIf(enabled) {
		if (!enabled) return this.kill()
	}

	bindings.on = bindingOn
	function bindingOn(ev, fn, a1, a2) {
		if (typeof fn == "string") {
			var _fn = fn
			fn = function(e) {
				Mediator.emit(_fn, e, this, a1, a2)
			}
		}
		this.on(ev, fn)
	}

	bindings["with"] = bindingWith
	function bindingWith(map) {
		return this.render(Object.merge(El.scope(this, true), map))
	}

	bindings.emitForm = emitForm
	function emitForm(ev, a1, a2, a3, a4) {
		this.on("submit", function(e) {
			var data = JSON.serializeForm(this)
			Mediator.emit(ev, e, data, a1, a2, a3, a4)
			return false
		})
	}

	function getChilds(node) {
		var child
		, childs = node._childs
		if (!childs) {
			for (node._childs = childs = []; child = node.firstChild;) {
				childs.push(child);
				node.removeChild(child)
			}
		}
		return childs
	}

	bindings.each = bindingsEach

	function bindingsEach(data, expr) {
		var node = this
		, child = getChilds(node)[0]
		, match = /^\s*(\w+) in (\w*)(.*)/.exec(expr)
		, fn = "with(data){var out=[],loop={i:0,offset:0},_1,_2=" + match[2]
		+ match[3].replace(/ (limit|offset):\s*(\d+)/ig, ";loop.$1=$2")
		+ ";if(_2)for(_1 in _2)if(hasOwn.call(_2,_1)&&!(loop.offset&&loop.offset--)){"
		+     "loop.i++;"
		+     "if(loop.limit&&loop.i-loop.offset>loop.limit)break;"
		+     "var clone=el.cloneNode(true)"
		+     ",scope=El.scope(clone,data);"
		+     "scope.loopKey=loop.key=_1;"
		+     "scope.loop=loop;"
		+     "scope." + match[1] + "=_2[_1];"
		+     "out.push(clone);"
		+ "};return out}"

		var childs = Function("hasOwn,el,data", fn)(hasOwn, child, data)

		node.empty().append(childs).render()
		return node
	}
}(El.bindings)

