


/*
* @version    0.1.5
* @date       2015-07-02
* @stability  1 - Experimental
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



!function(root) {
	var arrProto = Array.prototype

	function Model(data) {
		var model = this
		model.data = {}
		model.lists = []
		if (model.init) model.init(data)
		model.set(data)
	}

	Object.merge(Model.prototype, Event.Emitter, {
		set: function(key, val) {
			var model = this
			, changed = []
			, data = key

			if (typeof key != "object") {
				JSON.pointer(data = {}, key, val)
			}

			if (model.validate(data)) {
				JSON.mergePatch(model.data, data, changed)
			}

			if (changed.length) {
				model.emit("change", changed, model)
				changed.map(function(pointer) {
					model.emit("change:" + pointer, JSON.pointer(model.data, pointer))
				})
				model.lists.map(function(list) {
					list.emit("change", changed, model)
				})
			}
			return changed
		},
		get: function(name) {
			if (name.charAt(0) == "/") {
				return JSON.pointer(this.data, name)
			}
			return this.data[name]
		},
		validate: returnTrue,
		destroy: function(next) {
			var model = this
			, arr = model.lists.slice()
			, len = arr.length

			for (; len--; ) arr[len].remove(model)

			model.emit("destroy")
			model.off()
			//delete model.constructor.cached[model.get("id")]
		},
		toJSON: function() {
			return this.data
		}
	})

	root.Model = Model.cache(true, function(data) {
		return data.id
	})

	function List(name, _a1, _a2) {
		var list = this
		list.name = name
		if (list.init) list.init(name, _a1, _a2)
	}

	Object.merge(List.prototype = [], Event.Emitter, {
		wait: Fn.hold,
		syncMethods: [
			"add", "emit", "filterFn", "merge",
			"model", "pluck", "remove", "removeAll", "sortFn"
		],
		model: root.Model,
		add: function(item) {
			var pos
			, list = this

			if (!item) return list

			if (!(item instanceof Model)) {
				item = list.model(item)
			}

			if (item.lists.indexOf(list) == -1 && list.filterFn(item.data)) {
				item.lists.push(list)
				pos = indexFor(list, item, list.sortFn)
				list.splice(pos , 0, item)
				list.emit("add", item, pos)
			}
			return list
		},
		filterFn: returnTrue,
		remove: function(item) {
			var list = this
			if (item && item.lists && item.lists.remove(list) > -1) {
				list.emit("remove", item, arrProto.remove.call(list, item))
			}
			return list
		},
		removeAll: function(){
			var list = this, len = list.length
			while (len--) list.remove( list[len] )
			return list
		},
		each: function(fn, scope) {
			arrProto.each.call(this, fn, scope)
			return this
		},
		get: function(id, next, scope) {
			var list = this
			, item = list.model.cached[id]

			if (item && item.lists.indexOf(list) != -1) {
				return next ?
					(next.call(scope || list, null, item), list) :
					item
			}

			next && next.call(scope || list, Error("Item not found"))
		},
		at: function(index, next, scope) {
			var list = this
			, item = list[index]
			, err = item ? null : Error("Item not found")

			return next ?
				(next.call(scope || list, err, item), list) :
				item
		},
		first: function(next, scope) {
			return this.at(0, next, scope)
		},
		last: function(next, scope) {
			return this.at(this.length - 1, next, scope)
		},
		then: function(next, scope) {
			next.call(scope || this)
			return this
		},
		onEmpty: function(next, scope) {
			if (!this.length) next.call(scope || this)
			return this
		},
		pluck: function(field, next, scope) {
			var list = this
			, arr = arrProto.map.call(list, function(item) {
				return item.get(field)
			})
			return next ?
				(next.call(scope || list, arr), list) :
				arr
		},
		set: function(key, val) {
			this.each(function(item) {
				item.set(key, val)
			})
			return this
		},
		merge: function(mergeList) {
			var list = this
			, lists = list.lists || (list.lists = [])

			if (mergeList && lists.indexOf(mergeList) == -1) {
				lists.push(mergeList)

				mergeList
				.each(list.add, list)
				.on("add", list.add, list)
				.on("remove", _remove, list)
				.on("change", _change, list)
				.then(list.wait())
			}

			return list
		},
		reFilter: function() {
			var list = this
			, lists = list.lists || (list.lists = [])

			lists.each(function(parent) {
				parent.each(function(item) {
					_change.call(list, null, item)
				})
			})
		},
		toJSON: function() {
			return this.slice(0)
		}
	})

	function _remove(item) {
		if (item.lists.every(notIn, this.lists)) this.remove(item)
	}

	function _change(changed, item) {
		var list = this
		, matches = list.filterFn(item.data)
		, inList = item.lists.indexOf(list) > -1

		if (matches && !inList) list.add(item)
		if (!matches && inList) list.remove(item)
	}


	root.List = List.cache(true, function(name) {
		return name
	}, {})



	function returnTrue() {
		return true
	}

	function notIn(v) {
		return this.indexOf(v) == -1
	}

	function indexFor(arr, el, compFn) {
		var o
		, i = 0
		, l = arr.length

		// Fast lookup for last position
		if (compFn && l && compFn(el, arr[l-1]) < 1) {
			// Otherwise binary search
			for (; i<l; ) compFn(el, arr[o=(i+l)>>1]) < 0 ? l=o : i=o+1
		}
		return l
	}

}(this)



